@@ -27,6 +27,7 @@ class Config(object):
         loglevel='info',
         pidfile=None,
         proc_name = None,
+        spew=False,
         timeout=30,
         tmp_upload_dir=None,
         umask="0",
@@ -60,9 +61,9 @@ class Config(object):
         if not os.path.exists(self.config_file):
             return {}
 
-        config = {}
+        config = {} 
         try:
-            execfile(self.config_file, config)
+            execfile(self.config_file, globals(), config)
         except:
             sys.exit("Could not read config file %r" % (self.config_file,))
 
