@@ -57,6 +57,7 @@ class Worker(object):
         signal.signal(signal.SIGUSR1, self.handle_usr1)
         signal.signal(signal.SIGTERM, self.handle_exit)
         signal.signal(signal.SIGINT, self.handle_exit)
+        signal.signal(signal.SIGWINCH, self.handle_winch)
         
     def handle_usr1(self, sig, frame):
         self.nr = -65536;
@@ -70,6 +71,10 @@ class Worker(object):
 
     def handle_exit(self, sig, frame):
         sys.exit(0)
+
+    def handle_winch(self, sig, fname):
+        # Ignore SIGWINCH in worker. Fix crash on OpenBSD
+        return
         
     def notify(self):
         """\
@@ -97,12 +102,17 @@ class Worker(object):
     def accept(self):
         try:
             client, addr = self.socket.accept()
+            self.init_sock(client)
             self.handle(client, addr)
             self.nr += 1
         except socket.error, e:
             if e[0] not in (errno.EAGAIN, errno.ECONNABORTED):
                 raise
     
+    def init_sock(self, sock):
+        sock.setblocking(1)
+        util.close_on_exec(sock)
+
     def run(self):
         self.init_process()
         self.nr = 0
@@ -145,12 +155,22 @@ class Worker(object):
                 raise
 
     def handle(self, client, addr):
-        util.close_on_exec(client)
         try:
             req = http.Request(client, addr, self.address, self.conf)
 
             try:
-                response = self.app(req.read(), req.start_response)
+                environ = req.read()
+                if not environ or not req.parser.status_line:
+                    return
+
+                respiter = self.app(environ, req.start_response)
+                for item in respiter:
+                    req.response.write(item)
+                req.response.close()
+                if hasattr(respiter, "close"):
+                    respiter.close()
+            except socket.error:
+                raise
             except Exception, e:
                 # Only send back traceback in HTTP in debug mode.
                 if not self.debug:
@@ -158,7 +178,6 @@ class Worker(object):
                 util.write_error(client, traceback.format_exc())
                 return 
 
-            http.Response(client, response, req).send()
         except socket.error, e:
             if e[0] != errno.EPIPE:
                 self.log.exception("Error processing request.")
@@ -176,4 +195,3 @@ class Worker(object):
                 pass
         finally:    
             util.close(client)
-            
